@@ -4,7 +4,7 @@ from __future__ import annotations
 import logging
 from multiprocessing import cpu_count
 from typing import Sequence, Callable, Literal
-from .levels import CalibratedData, read_step, _ReductionStep, RawData
+from .levels import CalibratedData, read_step, _ReductionStep, RawData, CombinedData
 from . import types as tp
 import attr
 import h5py
@@ -24,6 +24,8 @@ import inspect
 import p_tqdm
 from pathlib import Path
 from .averaging import weighted_mean
+import datetime
+from edges_io.utils import ymd_to_jd
 
 logger = logging.getLogger(__name__)
 
@@ -38,7 +40,7 @@ def get_step_filter(filt: str) -> Callable:
 
 
 def step_filter(
-    axis: Literal["time", "freq", "both"],
+    axis: Literal["time", "freq", "day", "all"],
     multi_data: bool = False,
     data_type: type[_ReductionStep] | tuple[_ReductionStep] = _ReductionStep,
 ):
@@ -101,6 +103,8 @@ def step_filter(
 
             # Read all the data, in case they haven't been turned into objects yet.
             # And check that everything is the right type.
+            if not hasattr(data, "__len__"):
+                data = [data]
             data = [read_step(d) for d in data]
             assert all(isinstance(d, data_type) for d in data)
 
@@ -141,7 +145,12 @@ def step_filter(
                 )
 
             for n, d, f, this_f in zip(pre_flag_n, data, flg, this_flag):
-                f |= this_f if axis in ("both", "freq") else np.atleast_2d(this_f).T
+                if axis in ("all", "freq"):
+                    f |= this_f
+                elif axis == "day":
+                    f[this_f] = True
+                elif axis == "time":
+                    f[..., this_f, :] = True
 
                 if np.all(f):
                     logger.warning(
@@ -667,11 +676,9 @@ class TotalPowerAggregator(FrequencyAggregator):
             .fit(ydata=np.nanmean(fiducial_spectrum, axis=1))
             .fit
         )
-        init_flags = (
+        return (
             np.abs(standard_model(gha) - metric) / standard_model(gha)
         ) > self.init_threshold
-        print(np.shape(init_flags), len(np.where(init_flags)))
-        return init_flags
 
     def aggregate_file(self, data: CalibratedData) -> np.ndarray:
         """Compute the total power over frequency for each integration in a file."""
@@ -902,22 +909,32 @@ def _rfi_filter_factory(method: str):
         data: _ReductionStep,
         flags: np.ndarray[bool],
         n_threads: int = cpu_count(),
+        freq_range: tuple[float, float] = (40, 200),
         **kwargs,
     ) -> np.ndarray:
 
-        return tools.run_xrfi(
+        mask = (data.raw_frequencies >= freq_range[0]) & (
+            data.raw_frequencies <= freq_range[1]
+        )
+
+        out_flags = tools.run_xrfi(
             method=method,
-            spectrum=data.spectrum,
-            freq=data.raw_frequencies,
-            flags=flags,
-            weights=data.weights,
+            spectrum=data.spectrum[..., mask],
+            freq=data.raw_frequencies[mask],
+            flags=flags[..., mask],
+            weights=data.weights[..., mask],
             n_threads=n_threads,
             **kwargs,
         )
 
+        out = np.zeros_like(flags)
+        out[..., mask] = out_flags
+
+        return out
+
     fnc.__name__ = f"rfi_{method}_filter"
 
-    return step_filter(axis="both")(fnc)
+    return step_filter(axis="all")(fnc)
 
 
 rfi_model_filter = _rfi_filter_factory("model")
@@ -1062,7 +1079,6 @@ def _peak_power_filter(
         axis=1,
     )
     peak_power = 10 * np.log10(peak_power / mean)
-    print(peak_power.shape)
     return peak_power > threshold
 
 
@@ -1154,6 +1170,49 @@ def maxfm_filter(*, data: CalibratedData, threshold: float = 200):
     return maxfm > threshold
 
 
+@step_filter(axis="time", data_type=(RawData, CalibratedData))
+def rmsf_filter(
+    *,
+    data: CalibratedData | RawData,
+    threshold: float = 200,
+    freq_range: tuple[float, float] = (60, 80),
+    tload: float = 1000,
+    tcal: float = 300,
+):
+    """
+    Rmsf filter - filters out based on rms calculated between 60 and 80 MHz.
+
+    An initial powerlaw model is calculated using the normalized frequency range.
+    Data between the freq_range is clipped.
+    A standard deviation is calculated using the data and the init_model.
+    Then rms is calculated from the mean that is eatimated
+    using the standard deviation times initmodel.
+    """
+    freq_mask = (data.raw_frequencies >= freq_range[0]) & (
+        data.raw_frequencies <= freq_range[1]
+    )
+
+    if not np.any(freq_mask):
+        return np.zeros(len(data.spectrum), dtype=bool)
+
+    semi_calibrated_data = (data.spectrum * tload) + tcal
+    freq = data.raw_frequencies[freq_mask]
+    init_model = (freq / 75.0) ** -2.5
+
+    T75 = np.sum(init_model * semi_calibrated_data[:, freq_mask], axis=1) / np.sum(
+        init_model ** 2
+    )
+
+    rms = np.sqrt(
+        np.mean(
+            (semi_calibrated_data[:, freq_mask] - np.outer(T75, init_model)) ** 2,
+            axis=1,
+        )
+    )
+
+    return rms > threshold
+
+
 @step_filter(axis="time", data_type=(RawData, CalibratedData))
 def filter_150mhz(*, data: RawData | CalibratedData, threshold: float):
     """Filter based on power around 150 MHz.
@@ -1175,3 +1234,50 @@ def filter_150mhz(*, data: RawData | CalibratedData, threshold: float):
     d = 200.0 * np.sqrt(rms) / av
 
     return d > threshold
+
+
+@step_filter(axis="time", data_type=(RawData,))
+def power_percent_filter(
+    *,
+    data: RawData,
+    freq_range: tuple[float, float] = (100, 200),
+    min_threshold: float = -0.7,
+    max_threshold: float = 3,
+):
+    """Filter for the power above 100 MHz seen in swpos 0.
+
+    Calculates the percentage of power between 100 and 200 MHz
+    & when the switch is in position 0.
+    And flags integrations if the percentage is above or below the given threshold.
+    """
+    p0 = data.spectra["switch_powers"][0]
+
+    mask = (data.raw_frequencies > freq_range[0]) & (
+        data.raw_frequencies <= freq_range[1]
+    )
+
+    if not np.any(mask):
+        return np.zeros(len(data.spectrum), dtype=bool)
+
+    ppercent = 100 * np.sum(p0[:, mask], axis=1) / np.sum(p0, axis=1)
+    return (ppercent < min_threshold) | (ppercent > max_threshold)
+
+
+@step_filter(axis="day", data_type=(CombinedData,))
+def day_filter(
+    *, data: CombinedData, dates: Sequence[datetime.datetime | tuple[int, int]]
+):
+    """Filter out specific days."""
+    filter_dates = []
+    for date in dates:
+        if isinstance(date, datetime.date):
+            date = (date.year, ymd_to_jd(date.year, date.month, date.day))
+            filter_dates.append(date)
+        elif len(date) == 3:
+            filter_dates.append(tuple(date)[:2])
+        elif len(date) == 2:
+            filter_dates.append(tuple(date))
+        else:
+            raise ValueError(f"date '{date}' cannot be parsed as a date.")
+
+    return np.array([date[:2] in filter_dates for date in data.dates])
